% BFGS
%
% ff is the function to optimize
%     called as "[ value, gradient ] = ff( xx )"
% initial_xx is the starting point
% initial_inverse_hessian is the initial Hessian (usually I)
% line_search is the line search routine
%     called as "tt = line_search( ff, xx, direction )"
% epsilon is the stopping threshold
%
% xx is the final iterate
% values is the vector of function values (one for each iterate)
% runtimes is the vector of cumulative runtimes (one for each iterate)
% xxs is the matrix of iterates
%
function [ xx, values, runtimes, xxs ] = algorithm_bfgs( ff, initial_xx, initial_inverse_hessian, line_search, epsilon )

maximum_iterations = 65536;

mm = length( initial_xx );
if ( any( size( initial_inverse_hessian ) ~= [ mm, mm ] ) ); error( 'dimensions are not compatible' ); end
if ( epsilon <= 0.0 ); error( 'epsilon must be positive' ); end

values = [];
runtimes = [];
xxs = [];

tic;

xx = initial_xx;
inverse_hessian = initial_inverse_hessian;

iterations = 0;
while true

	[ value, gradient ] = ff( xx );
	gradient_norm_squared = gradient' * gradient;

	% only compute the necessary return values
	if ( nargout > 1 )
		values = [ values, value ];
		if ( nargout > 2 )
			runtimes = [ runtimes, toc ];
			if ( nargout > 3 )
				xxs = [ xxs, xx ];
			end
		end
	end

	if ( gradient_norm_squared <= epsilon ); break; end

	if ( iterations > 0 )

		ss = xx - old_xx;
		yy = gradient - old_gradient;
		tau = 1.0 / ( yy' * ss );
		inverse_hessian = ( eye( mm ) - tau * ss * yy' ) * inverse_hessian * ( eye( mm ) - tau * yy * ss' ) + tau * ss * ss';

	end

	direction = -inverse_hessian * gradient;

	tt = line_search( ff, xx, direction );

	old_xx = xx;
	old_gradient = gradient;
	xx = xx + tt * direction;

	iterations = iterations + 1;
	if ( iterations > maximum_iterations ); error( 'too many iterations' ); end

end
