% "Exact" line search (using bisection method)
%
% ff is the function to optimize
%     called as "[ value, gradient ] = ff( xx )"
% xx is the current iterate
% direction is the direction along which to perform the line search
% epsilon is the maximum allowed error in the resulting step size tt
%
% tt is the resulting step length
%
function tt = line_search_bisection( ff, xx, direction, epsilon )

maximum_iterations = 65536;

% check parameter bounds
if ( epsilon <= 0.0 ); error( 'epsilon must be positive' ); end

[ value, gradient ] = ff( xx );
derivative = direction' * gradient;

if ( derivative >= 0.0 )

	% if we're not given a descent direction, we shouldn't go anywhere
	tt = 0.0;

else

	iterations = 0;
	tt_high = 1.0;
	[ value, gradient ] = ff( xx + tt_high * direction );
	while ( direction' * gradient < 0 )

		tt_high = tt_high * 2;
		[ value, gradient ] = ff( xx + tt_high * direction );

		iterations = iterations + 1;
		if ( iterations >= maximum_iterations ); error( 'too many iterations' ); end

	end

	iterations = 0;
	tt = 0.0;
	while ( tt_high - tt > epsilon )

		tt_middle = 0.5 * ( tt + tt_high );
		[ value, gradient ] = ff( xx + tt_middle * direction );

		if ( direction' * gradient < 0 )
			tt = tt_middle;
		else
			tt_high = tt_middle;
		end

		iterations = iterations + 1;
		if ( iterations >= maximum_iterations ); error( 'too many iterations' ); end

	end

	tt = 0.5 * ( tt + tt_high );

end
