% read the dataset
% logistic_small contains 1000 10-dimensional examples
% logistic_large contains 1000 50-dimensional examples
%
%data = dlmread( 'logistic_small.csv', ',' );
data = dlmread( 'logistic_large.csv', ',' );

% extract the labels yy and features XX from the dataset
%
yy = data( :, 1 );
XX = data( :, 2:size( data, 2 ) );
[ nn, mm ] = size( XX );

% use the logistic objective. this notation defines a "closure", returning a
% function which takes xx as its only parameter, and calls objective_logistic
% with parameters yy and XX defined above, and xx (the parameter of the
% closure)
%
ff = @( xx ) objective_logistic( yy, XX, xx );

% use backtracking line search. this defines a closure which takes three
% parameters. note that the alpha and beta parameters to the backtracking line
% search are 0.4 and 0.9, respectively
%
line_search = @( ff, xx, direction ) line_search_backtracking( ff, xx, direction, 0.4, 0.9 );

% start at 0, with (for BFGS) an identity inverse Hessian
%
initial_xx = zeros( mm, 1 );
initial_inverse_hessian = eye( mm );

epsilon = 1e-4;


% now we create the plot

[ xx, values ] = algorithm_newton( ff, initial_xx, line_search, 1e-8 );
minimum_ff = min( values );

[ xx, values, runtimes ] = algorithm_gd( ff, initial_xx, line_search, epsilon );
handle_gd = semilogy( runtimes, values - min( values ), '-or' );

hold on;

[ xx, values, runtimes ] = algorithm_cg( ff, initial_xx, line_search, epsilon );
handle_cg = semilogy( runtimes, values - min( values ), '-og' );

[ xx, values, runtimes ] = algorithm_bfgs( ff, initial_xx, initial_inverse_hessian, line_search, epsilon );
handle_bfgs = semilogy( runtimes, values - min( values ), '-ob' );

[ xx, values, runtimes ] = algorithm_newton( ff, initial_xx, line_search, epsilon );
handle_newton = semilogy( runtimes, values - min( values ), '-om' );

xlabel( 'time' )
ylabel( 'suboptimality' )
legend( [ handle_gd, handle_cg, handle_bfgs, handle_newton ], 'GD', 'CG', 'BFGS', 'Newton' )

hold off;
